@@ -12,8 +12,6 @@ from ford._markdown import MetaMarkdown
 import ford.sourceform
 
 from itertools import chain
-from os.path import relpath
-import pathlib
 
 import pytest
 from bs4 import BeautifulSoup
@@ -1045,7 +1043,7 @@ def test_submodule_uses(copy_fortran_file):
 
 
 def test_make_links(copy_fortran_file):
-    links = "[[a]] [[b(type)]] [[b:c]] [[a:d]] [[b:e]] [[f(proc)]] [[a:g]] [[h]]"
+    links = "[[a]] [[b(type)]] [[b:c]] [[a:d]] [[b:e]] [[F(proc)]] [[A:G]] [[H]] [[I]]"
 
     data = f"""\
     module a !! {links}
@@ -1075,6 +1073,9 @@ def test_make_links(copy_fortran_file):
 
     program h !! {links}
     end program h
+
+    submodule (a) i !! {links}
+    end submodule i
     """
     settings = copy_fortran_file(data)
     project = create_project(settings)
@@ -1090,6 +1091,7 @@ def test_make_links(copy_fortran_file):
         "f": "../proc/f.html",
         "g": "../module/a.html#variable-g",
         "h": "../program/h.html",
+        "i": "../module/i.html",
     }
 
     for item in chain(
@@ -1104,7 +1106,75 @@ def test_make_links(copy_fortran_file):
         assert link_locations == expected_links, (item, item.name)
 
 
-def test_link_with_context(copy_fortran_file):
+def test_make_links_with_entity_spec(copy_fortran_file):
+    links = (
+        "[[a(module)]] [[b(type)]] [[b(type):c(variable)]] "
+        "[[A(MODULE):D(SUBROUTINE)]] [[B(TYPE):E]] [[F(PROC)]] "
+        "[[A(module):G(variable)]] [[H(program)]] [[I(SUBMODULE)]]"
+    )
+
+    data = f"""\
+    module a !! {links}
+      type b !! {links}
+        integer :: c !! {links}
+      contains
+        final :: d !! {links}
+        procedure :: e !! {links}
+      end type b
+
+      interface b !! {links}
+        module procedure f
+      end interface b
+
+      type(b) :: g !! {links}
+    contains
+      subroutine d(self) !! {links}
+        type(b) :: self !! {links}
+      end subroutine d
+      subroutine e(self) !! {links}
+        class(b) :: self !! {links}
+      end subroutine e
+      function f() !! {links}
+        type(b) :: f !! {links}
+      end function f
+    end module a
+
+    program h !! {links}
+    end program h
+
+    submodule (a) i !! {links}
+    end submodule i
+    """
+    settings = copy_fortran_file(data)
+    project = create_project(settings)
+    md = MetaMarkdown(project=project)
+    project.markdown(md)
+
+    expected_links = {
+        "a": "../module/a.html",
+        "b": "../type/b.html",
+        "c": "../type/b.html#variable-c",
+        "d": "../proc/d.html",
+        "e": "../type/b.html#boundprocedure-e",
+        "f": "../proc/f.html",
+        "g": "../module/a.html#variable-g",
+        "h": "../program/h.html",
+        "i": "../module/i.html",
+    }
+
+    for item in chain(
+        project.files[0].children,
+        project.types[0].children,
+        project.procedures[0].children,
+        project.procedures[2].children,
+    ):
+        docstring = BeautifulSoup(item.doc, features="html.parser")
+        link_locations = {a.string: a.get("href", None) for a in docstring("a")}
+
+        assert link_locations == expected_links, (item, item.name)
+
+
+def test_make_links_with_context(copy_fortran_file):
     data = """\
     module a !! [[g]]
       type b !! [[c]] [[e]] [[g]]
@@ -1115,7 +1185,7 @@ def test_link_with_context(copy_fortran_file):
 
       type(b) :: g
     contains
-      subroutine d(i) !! [[i]] [[f]] [[f:x]]
+      subroutine d(i) !! [[I]] [[F]] [[F:X]]
         type(b) :: i
         contains
           integer function f(x)
